# -*- coding: utf-8 -*-
"""

    mslib.mswms.mpl_vsec
    ~~~~~~~~~~~~~~~~~~~~

    Vertical section style super classes for use with the
    VerticalSectionDriver class.

    This file is part of mss.

    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
    :copyright: Copyright 2016-2021 by the mss team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
# style definitions should be put in mpl_vsec_styles.py

import PIL.Image
import io
import logging
import numpy as np
from abc import abstractmethod
from xml.dom.minidom import getDOMImplementation
import matplotlib as mpl
from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
import mpl_toolkits.axes_grid1

from mslib.mswms import mss_2D_sections
from mslib.utils.units import convert_to, units
from mslib.mswms.utils import make_cbar_labels_readable

mpl.rcParams['xtick.direction'] = 'out'
mpl.rcParams['ytick.direction'] = 'out'


class AbstractVerticalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
    """
    Abstract Vertical Section Style
    Superclass for all Matplotlib-based vertical section styles.
    """

    _pres_maj = np.concatenate([np.arange(top * 10, top, -top) for top in (10000, 1000, 100, 10)] + [[10]])
    _pres_min = np.concatenate([np.arange(top * 10, top, -top // 10) for top in (10000, 1000, 100, 10)] + [[10]])

    def __init__(self, driver=None):
        """
        Constructor.
        """
        super(AbstractVerticalSectionStyle, self).__init__(driver=driver)

    def add_colorbar(self, contour, label=None, tick_levels=None, width="3%", height="30%", cb_format=None, left=0.08,
                     right=0.95, top=0.9, bottom=0.14, fraction=0.05, pad=0.01, loc=1, tick_position="left"):
        if not self.noframe:
            self.fig.subplots_adjust(left=left, right=right, top=top, bottom=bottom)
            cbar = self.fig.colorbar(contour, fraction=fraction, pad=pad)
            cbar.set_label(label)
        else:
            axins1 = mpl_toolkits.axes_grid1.inset_locator.inset_axes(
                self.ax, width=width, height=height, loc=loc)
            self.fig.colorbar(contour, cax=axins1, orientation="vertical", ticks=tick_levels, format=cb_format)
            axins1.yaxis.set_ticks_position(tick_position)
            make_cbar_labels_readable(self.fig, axins1)

    def supported_crs(self):
        """
        Returns a list of the coordinate reference systems supported by
        this style.
        """
        return ["VERT:LOGP"]

    # TODO: the general setup should be a separate class as well
    def _latlon_logp_setup(self, orography=105000.):
        """
        General setup for lat/lon vs. log p vertical cross-sections.
        """
        ax = self.ax

        # Set xticks so that they display lat/lon. Plot "numlabels" labels.
        tick_index_step = max(1, len(self.lat_inds) // int(self.numlabels))
        ax.set_xticks(self.lat_inds[::tick_index_step])
        ax.set_xticklabels([f"{d[0]:2.1f}, {d[1]:2.1f}"
                            for d in zip(self.lats[::tick_index_step],
                                         self.lons[::tick_index_step])],
                           rotation=25, fontsize=10, horizontalalignment='right')

        if self.draw_verticals:
            # Add lines to highlight points if any are given.
            ipoint = 0
            for i, (lat, lon) in enumerate(zip(self.lats, self.lons)):
                if (ipoint < len(self.highlight) and
                    np.hypot(lat - self.highlight[ipoint][0],
                             lon - self.highlight[ipoint][1]) < 2E-10):
                    ax.axvline(i, color='k', linewidth=2, linestyle='--', alpha=0.5)
                    ipoint += 1

        # Add lower limit of pressure curtain to indicate orography.
        ax.fill_between(self.lat_inds, orography, y2=self.p_bot,
                        color=self.orography_color)

        # Set pressure axis scale to log.
        ax.set_yscale("log")

        major_ticks = self._pres_maj[(self._pres_maj <= self.p_bot) & (self._pres_maj >= self.p_top)]
        minor_ticks = self._pres_min[(self._pres_min <= self.p_bot) & (self._pres_min >= self.p_top)]
        labels = [f"{int(x / 100.)}" if (x / 100.) - int(x / 100.) == 0
                  else f"{float(x / 100.)}"
                  for x in major_ticks]
        if len(labels) > 20:
            labels = ["" if x.split(".")[-1][0] in "975" else x for x in labels]
        elif len(labels) > 10:
            labels = ["" if x.split(".")[-1][0] in "9" else x for x in labels]

        # Draw ticks and tick labels.
        ax.set_yticks(minor_ticks, minor=True)
        ax.set_yticks(major_ticks, minor=False)
        ax.set_yticklabels(labels, minor=False, fontsize=10)

        # Set axis limits and draw grid for major ticks.
        ax.set_xlim(self.lat_inds[0], self.lat_inds[-1])
        ax.set_ylim(self.p_bot, self.p_top)
        ax.grid(b=True)

    @abstractmethod
    def _plot_style(self):
        """
        Can call self._log_setup()
        """
        pass

    def plot_vsection(self, data, lats, lons, valid_time, init_time,
                      resolution=(-1, -1), bbox=(-1, 1050, -1, 200), style=None,
                      show=False,
                      highlight=None, noframe=False, figsize=(960, 480), draw_verticals=False,
                      numlabels=10, orography_color='k', transparent=False,
                      return_format="image/png"):
        """
        """
        # Check if required data is available.
        self.data_units = self.driver.data_units.copy()
        for datatype, dataitem, dataunit in self.required_datafields:
            if dataitem not in data:
                raise KeyError(f"required data field '{dataitem}' not found")
            origunit = self.driver.data_units[dataitem]
            if dataunit is not None:
                data[dataitem] = convert_to(data[dataitem], origunit, dataunit)
                self.data_units[dataitem] = dataunit
            else:
                logging.debug("Please add units to plot variables")

        # Copy parameters to properties.
        self.data = data
        self.lats = lats
        self.lat_inds = np.arange(len(lats))
        self.lons = lons
        self.valid_time = valid_time
        self.init_time = init_time
        self.resolution = resolution
        self.style = style
        self.highlight = highlight
        self.noframe = noframe
        self.draw_verticals = draw_verticals
        self.p_bot = bbox[1] * 100
        self.p_top = bbox[3] * 100
        self.numlabels = numlabels
        self.orography_color = orography_color

        # Provide an air_pressured 2-D field in 'Pa' from vertical axis
        if (("air_pressure" not in self.data) and
                units(self.driver.vert_units).check("[pressure]")):
            self.data_units["air_pressure"] = "Pa"
            self.data["air_pressure"] = convert_to(
                self.driver.vert_data[::-self.driver.vert_order, np.newaxis],
                self.driver.vert_units, self.data_units["air_pressure"]).repeat(
                    len(self.lats), axis=1)
        if (("air_potential_temperature" not in self.data) and
                units(self.driver.vert_units).check("[temperature]")):
            self.data_units["air_potential_temperature"] = "K"
            self.data["air_potential_temperature"] = convert_to(
                self.driver.vert_data[::-self.driver.vert_order, np.newaxis],
                self.driver.vert_units, self.data_units["air_potential_temperature"]).repeat(
                    len(self.lats), axis=1)

        # Derive additional data fields and make the plot.
        self._prepare_datafields()
        if "air_pressure" not in self.data:
            raise KeyError(
                "'air_pressure' need to be available for VSEC plots."
                "Either provide as data or compute in _prepare_datafields")

        # Code for producing a png image with Matplotlib.
        # ===============================================
        if return_format == "image/png":

            logging.debug("creating figure..")
            dpi = 80
            figsize = (figsize[0] / dpi), (figsize[1] / dpi)
            facecolor = "white"
            self.fig = mpl.figure.Figure(figsize=figsize, dpi=dpi, facecolor=facecolor)
            logging.debug("\twith frame and legends" if not noframe else
                          "\twithout frame")
            if noframe:
                self.ax = self.fig.add_axes([0.0, 0.0, 1.0, 1.0])
            else:
                self.ax = self.fig.add_axes([0.07, 0.17, 0.9, 0.72])

            # prepare horizontal axis
            self.horizontal_coordinate = self.lat_inds[np.newaxis, :].repeat(
                self.data["air_pressure"].shape[0], axis=0)

            self._plot_style()

            # Set transparency for the output image.
            if transparent:
                self.fig.patch.set_alpha(0.)

            # Return the image as png embedded in a StringIO stream.
            canvas = FigureCanvas(self.fig)
            output = io.BytesIO()
            canvas.print_png(output)

            if show:
                logging.debug("saving figure to mpl_vsec.png ..")
                canvas.print_png("mpl_vsec.png")

            # Convert the image to an 8bit palette image with a significantly
            # smaller file size (~factor 4, from RGBA to one 8bit value, plus the
            # space to store the palette colours).
            # NOTE: PIL at the current time can only create an adaptive palette for
            # RGB images, hence alpha values are lost here. If transparency is
            # requested, the figure face colour is stored as the "transparent"
            # colour in the image. This works in most cases, but might lead to
            # visible artefacts in some cases.
            logging.debug("converting image to indexed palette.")
            # Read the above stored png into a PIL image and create an adaptive
            # colour palette.
            output.seek(0)  # necessary for PIL.Image.open()
            palette_img = PIL.Image.open(output).convert(
                mode="RGB").convert("P", palette=PIL.Image.ADAPTIVE)
            output = io.BytesIO()
            if not transparent:
                logging.debug("saving figure as non-transparent PNG.")
                palette_img.save(output, format="PNG")  # using optimize=True doesn't change much
            else:
                # If the image has a transparent background, we need to find the
                # index of the background colour in the palette. See the
                # documentation for PIL's ImagePalette module
                # (http://www.pythonware.com/library/pil/handbook/imagepalette.htm). The
                # idea is to create a 256 pixel image with the same colour palette
                # as the original image and use it as a lookup-table. Converting the
                # lut image back to RGB gives us a list of all colours in the
                # palette. (Why doesn't PIL provide a method to directly access the
                # colours in a palette??)
                lut = palette_img.resize((256, 1))
                lut.putdata(list(range(256)))
                lut = [c[1] for c in lut.convert("RGB").getcolors()]
                facecolor_rgb = list(mpl.colors.hex2color(mpl.colors.cnames[facecolor]))
                for i in [0, 1, 2]:
                    facecolor_rgb[i] = int(facecolor_rgb[i] * 255)
                facecolor_index = lut.index(tuple(facecolor_rgb))

                logging.debug("saving figure as transparent PNG with transparency index %i.",
                              facecolor_index)
                palette_img.save(output, format="PNG", transparency=facecolor_index)

            logging.debug("returning figure..")
            return output.getvalue()

        # Code for generating an XML document with the data values in ASCII format.
        # =========================================================================
        elif return_format == "text/xml":

            impl = getDOMImplementation()
            xmldoc = impl.createDocument(None, "MSS_VerticalSection_Data", None)

            # Title of this section.
            node = xmldoc.createElement("Title")
            node.appendChild(xmldoc.createTextNode(self.title))
            xmldoc.documentElement.appendChild(node)

            # Time information of this section.
            node = xmldoc.createElement("ValidTime")
            node.appendChild(xmldoc.createTextNode(self.valid_time.strftime("%Y-%m-%dT%H:%M:%SZ")))
            xmldoc.documentElement.appendChild(node)

            node = xmldoc.createElement("InitTime")
            node.appendChild(xmldoc.createTextNode(self.init_time.strftime("%Y-%m-%dT%H:%M:%SZ")))
            xmldoc.documentElement.appendChild(node)

            # Longitude data.
            node = xmldoc.createElement("Longitude")
            node.setAttribute("num_waypoints", f"{len(self.lons)}")

            data_str = ""
            for value in self.lons:
                data_str += str(value) + ","
            data_str = data_str[:-1]

            node.appendChild(xmldoc.createTextNode(data_str))
            xmldoc.documentElement.appendChild(node)

            # Latitude data.
            node = xmldoc.createElement("Latitude")
            node.setAttribute("num_waypoints", f"{len(self.lats)}")

            data_str = ""
            for value in self.lats:
                data_str += str(value) + ","
            data_str = data_str[:-1]

            node.appendChild(xmldoc.createTextNode(data_str))
            xmldoc.documentElement.appendChild(node)

            # Variable data.
            data_node = xmldoc.createElement("Data")

            for var in self.data:
                node = xmldoc.createElement(var)
                data_shape = self.data[var].shape
                node.setAttribute("num_levels", f"{data_shape[0]}")
                node.setAttribute("num_waypoints", f"{data_shape[1]}")

                data_str = ""
                for data_row in self.data[var]:
                    for value in data_row:
                        data_str += str(value) + ","
                    data_str = data_str[:-1] + "\n"
                data_str = data_str[:-1]

                node.appendChild(xmldoc.createTextNode(data_str))
                data_node.appendChild(node)

            xmldoc.documentElement.appendChild(data_node)

            # Return the XML document as formatted string.
            return xmldoc.toprettyxml(indent="  ")



# -*- coding: ISO-8859-15 -*-
# Copyright (c) 2006, Ancient World Mapping Center
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the University of North Carolina nor the names of
#      its contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
#
#
#
# =============================================================================
# Copyright (c) 2004, 2006 Sean C. Gillies
# Copyright (c) 2005 Nuxeo SARL <http://nuxeo.com>
#
# Authors : Sean Gillies <sgillies@frii.com>
#           Julien Anguenot <ja@nuxeo.com>
#
# Contact email: sgillies@frii.com
#
# ******************************************************************************
# Changes made for the MSS (flagged with "(mss)"):
#   -- added 'Abstract' to Layer keywords (2010-08).
#   -- bugfix: copy parent SRS only if it has been defined (2010-09-22)
#   -- added dimensions and extents parsing (2011-01-13)
#   -- added storage of capabilities document (2011-01-17)
#   -- renamed to ogcwms (2017-04-28)
#   -- PEP8 review
#   -- adopted it to the recent 0.14 version https://pypi.python.org/pypi/OWSLib/0.14.0
# ******************************************************************************
#
# =============================================================================

"""
API for Web Map Service (WMS) methods and metadata.

Currently supports only versions 1.1.1/1.3.0 of the WMS protocol.
"""

from future import standard_library
standard_library.install_aliases()

import defusedxml.ElementTree as etree
import requests
import logging

from owslib.util import ServiceException
from owslib.etree import ParseError
from owslib.map import wms111, wms130, common
from owslib.util import ResponseWrapper, Authentication, strip_bom
from mslib.utils.config import config_loader


def openURL(url_base, data=None, method='Get', cookies=None,
            username=None, password=None,
            timeout=config_loader(dataset="WMS_request_timeout"),
            headers=None, verify=None, cert=None, auth=None, proxies=None):
    # (mss) added proxies
    # (mss) timeout default of 30secs set by the config_loader
    """
    Function to open URLs.

    Uses requests library but with additional checks for OGC service exceptions and url formatting.
    Also handles cookies and simple user password authentication.
    """
    headers = headers if headers is not None else {}
    rkwargs = {}

    rkwargs['timeout'] = timeout

    if auth:
        if username:
            auth.username = username
        if password:
            auth.password = password
        if cert:
            auth.cert = cert
        verify = verify and auth.verify
    else:
        auth = Authentication(username, password, cert, verify)
    if auth.username and auth.password:
        rkwargs['auth'] = (auth.username, auth.password)
    rkwargs['cert'] = auth.cert
    rkwargs['verify'] = verify

    # FIXUP for WFS in particular, remove xml style namespace
    # @TODO does this belong here?
    method = method.split("}")[-1]

    if method.lower() == 'post':
        try:
            etree.fromstring(data)
            headers['Content-Type'] = 'text/xml'
        except (ParseError, UnicodeEncodeError) as error:
            # (mss)
            logging.debug("ParseError, UnicodeEncodeError %s", error)

        rkwargs['data'] = data

    elif method.lower() == 'get':
        rkwargs['params'] = data

    else:
        raise ValueError(f"Unknown method ('{method}'), expected 'get' or 'post'")

    if cookies is not None:
        rkwargs['cookies'] = cookies

    req = requests.request(method.upper(),
                           url_base,
                           headers=headers,
                           # MSS
                           proxies=proxies,
                           **rkwargs)

    if req.status_code in [400, 401]:
        raise ServiceException(req.text)

    if req.status_code in [404, 500, 502, 503, 504]:  # add more if needed
        req.raise_for_status()

    # check for service exceptions without the http header set
    if 'Content-Type' in req.headers and req.headers['Content-Type'] in [
            'text/xml', 'application/xml', 'application/vnd.ogc.se_xml', 'application/vnd.ogc.wms_xml']:
        # just in case 400 headers were not set, going to have to read the xml to see if it's an exception report.
        se_tree = etree.fromstring(req.content)

        # to handle the variety of namespaces and terms across services
        # and versions, especially for "legacy" responses like WMS 1.3.0
        possible_errors = [
            '{http://www.opengis.net/ows}Exception',
            '{http://www.opengis.net/ows/1.1}Exception',
            '{http://www.opengis.net/ogc}ServiceException',
            'ServiceException'
        ]

        for possible_error in possible_errors:
            serviceException = se_tree.find(possible_error)
            if serviceException is not None:
                # and we need to deal with some message nesting
                raise ServiceException(
                    '\n'.join([str(t).strip() for t in serviceException.itertext() if t.strip()]))

    return ResponseWrapper(req)


class WebMapService(wms111.WebMapService_1_1_1):
    """Abstraction for OGC Web Map Service (WMS).

    Implements IWebMapService.
    """

    def __init__(self, url, version=None, xml=None, username=None, password=None,
                 parse_remote_metadata=False, headers=None,
                 timeout=config_loader(dataset="WMS_request_timeout"),
                 auth=None):
        """Initialize."""

        if auth:
            if username:
                auth.username = username
            if password:
                auth.password = password
        self.url = url
        self.version = version
        self.timeout = timeout
        self.headers = headers
        self._capabilities = None
        self.auth = auth or Authentication(username, password)

        # Authentication handled by Reader
        reader = WMSCapabilitiesReader(self.version, url=self.url, headers=headers, auth=self.auth)
        if xml:
            # read from stored xml
            self._capabilities = reader.readString(xml)
        else:
            # read from server
            self._capabilities = reader.read(self.url, timeout=self.timeout)

        self.request = reader.request
        if not self.version:
            self.version = self._capabilities.attrib["version"]
            if self.version not in ["1.1.1", "1.3.0"]:
                self.version = "1.1.1"
            reader.version = self.version

        self.WMS_NAMESPACE = "{http://www.opengis.net/wms}" if self.version == "1.3.0" else ""
        self.OGC_NAMESPACE = "{http://www.opengis.net/ogc}" if self.version == "1.3.0" else ""

        # avoid building capabilities metadata if the
        # response is a ServiceExceptionReport
        se = self._capabilities.find('ServiceException')
        if se is not None:
            err_message = str(se.text).strip()
            raise ServiceException(err_message)

        # (mss) Store capabilities document.
        self.capabilities_document = reader.capabilities_document
        # (mss)

        # build metadata objects
        self._buildMetadata(parse_remote_metadata)

    def _buildMetadata(self, parse_remote_metadata=False):
        ''' set up capabilities metadata objects '''

        # serviceIdentification metadata
        serviceelem = self._capabilities.find(f'{self.WMS_NAMESPACE}Service')
        self.identification = (wms130 if self.version == "1.3.0" else wms111)\
            .ServiceIdentification(serviceelem, self.version)

        # serviceProvider metadata
        self.provider = (wms130 if self.version == "1.3.0" else wms111).ServiceProvider(serviceelem)

        # serviceOperations metadata
        self.operations = []
        for elem in self._capabilities.find(f'{self.WMS_NAMESPACE}Capability/{self.WMS_NAMESPACE}Request')[:]:
            self.operations.append((wms130 if self.version == "1.3.0" else wms111).OperationMetadata(elem))

        # serviceContents metadata: our assumption is that services use a top-level
        # layer as a metadata organizer, nothing more.
        self.contents = {}
        caps = self._capabilities.find(f'{self.WMS_NAMESPACE}Capability')

        def gather_layers(parent_elem, parent_metadata):
            layers = []
            for index, elem in enumerate(parent_elem.findall(f'{self.WMS_NAMESPACE}Layer')):
                cm = ContentMetadata(elem, parent=parent_metadata,
                                     index=index + 1,
                                     parse_remote_metadata=parse_remote_metadata,
                                     version=self.version)
                if cm.id:
                    if cm.id in self.contents:
                        logging.debug('Content metadata for layer "%s" already exists. Using child layer' % cm.id)
                    layers.append(cm)
                    self.contents[cm.id] = cm
                cm.children = gather_layers(elem, cm)
            return layers
        gather_layers(caps, None)

        # for elem in caps.findall(f'{self.WMS_NAMESPACE}Layer'):
        #    cm = ContentMetadata(elem, version=self.version)
        #    self.contents[cm.id] = cm
        #    for subelem in elem.findall(f'{self.WMS_NAMESPACE}Layer'):
        #        subcm = ContentMetadata(subelem, cm, version=self.version)
        #        self.contents[subcm.id] = subcm

        # exceptions
        self.exceptions = [f.text for f in self._capabilities.findall(
            f'{self.WMS_NAMESPACE}Capability/{self.WMS_NAMESPACE}Exception/{self.WMS_NAMESPACE}Format')]

    @property
    def getcapabilities(self):
        """ Request and return capabilities document from the WMS as a
            file-like object.
            NOTE: this is effectively redundant now
        """
        reader = WMSCapabilitiesReader(self.version, url=self.url, auth=self.auth)
        u = self._open(reader.capabilities_url(self.url))
        # check for service exceptions, and return
        if u.info()['Content-Type'] == 'application/vnd.ogc.se_xml':
            se_xml = u.read()
            se_tree = etree.fromstring(se_xml)
            err_message = str(se_tree.find(f'{self.OGC_NAMESPACE}ServiceException').text).strip()
            raise ServiceException(err_message, se_xml)
        return u

    def getfeatureinfo(self):
        raise NotImplementedError


def ContentMetadata(elem, parent=None, children=None, index=0,
                    parse_remote_metadata=False,
                    timeout=config_loader(dataset="WMS_request_timeout"),
                    auth=None, version="1.3.0"):
    WMS_NAMESPACE = "{http://www.opengis.net/wms}" if version == "1.3.0" else ""

    if version == "1.3.0":
        metadata = wms130.ContentMetadata(elem, parent=parent, children=children, index=index,
                                          parse_remote_metadata=parse_remote_metadata, timeout=timeout, auth=auth)
    else:
        metadata = wms111.ContentMetadata(elem, parent=parent, children=children, index=index,
                                          parse_remote_metadata=parse_remote_metadata, timeout=timeout, auth=auth)

    # (mss) Parse dimensions and their extents.
    metadata.dimensions = {}
    metadata.extents = {}
    for dim in elem.findall(f'{WMS_NAMESPACE}Dimension'):
        dimname = dim.attrib.get("name").lower()
        metadata.dimensions[dimname] = dim.attrib
        if version == "1.3.0":
            metadata.extents[dimname] = dim.attrib
            metadata.extents[dimname]["values"] = dim.text.strip().split(",")
    if version == "1.1.1":
        for extent in elem.findall(f'{WMS_NAMESPACE}Extent'):
            extname = extent.attrib.get("name").lower()
            metadata.extents[extname] = extent.attrib
            if extent.text:
                metadata.extents[extname]["values"] = extent.text.strip().split(",")
            else:
                metadata.extents[extname]["values"] = []
    # (mss)

    # (mss) Added "Abstract".
    for key in ('Name', 'Title', 'Abstract'):
        val = elem.find(WMS_NAMESPACE + key)
        # (mss) Added " and val.text is not None".
        if val is not None and val.text is not None:
            setattr(metadata, key.lower(), val.text.strip())
        else:
            setattr(metadata, key.lower(), None)
        metadata.id = metadata.name  # conform to new interface

    # (mss) Replace owslib ContentMetadata children with ogcwms ContentMetadata
    metadata.layers = []
    for child in elem.findall('Layer'):
        metadata.layers.append(ContentMetadata(child, metadata, version=version))

    return metadata


class WMSCapabilitiesReader(common.WMSCapabilitiesReader):
    """Read and parse capabilities document into a lxml.etree infoset
    """

    def __init__(self, version='1.3.0', url=None, un=None, pw=None, headers=None, auth=None):
        """Initialize"""
        super().__init__(version, url, un, pw, headers, auth)
        # (mss) Store capabilities document.
        self.capabilities_document = None
        # (mss)

    def read(self, service_url,
             timeout=config_loader(dataset="WMS_request_timeout")):
        """Get and parse a WMS capabilities document, returning an
        elementtree instance

        service_url is the base url, to which is appended the service,
        version, and request parameters
        """
        getcaprequest = self.capabilities_url(service_url)

        # Don't specify a version if it is to be determined
        getcaprequest = getcaprequest.replace("&version=None", "").replace("?version=None", "")

        proxies = config_loader(dataset="proxies")

        # now split it up again to use the generic openURL function...
        spliturl = getcaprequest.split('?')
        u = openURL(spliturl[0], spliturl[1], method='Get', auth=self.auth, proxies=proxies)

        # (mss) Store capabilities document.
        self.capabilities_document = strip_bom(u.read())
        return etree.fromstring(self.capabilities_document)
        # (mss)

    def readString(self, st):
        """Parse a WMS capabilities document, returning an elementtree instance
                string should be an XML capabilities document
                """
        if not isinstance(st, str):
            raise ValueError("String must be of type string, not %s" % type(st))
        return etree.fromstring(st)


def removeXMLNamespace(tree):
    for elem in tree.iter():
        if elem.tag.startswith("{"):
            elem.tag = elem.tag.split("}")[-1]



# -*- coding: utf-8 -*-
"""

    mslib.mswms._tests.test_mss_plot_driver
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module provides pytest functions to tests mslib.mswms.mss_plot_driver

    This file is part of mss.

    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
    :copyright: Copyright 2017 Reimar Bauer, Joern Ungermann
    :copyright: Copyright 2016-2021 by the mss team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from datetime import datetime
import os
import warnings
import sys

import pytest
from PIL import Image
from xml.etree import ElementTree
import io
from mslib.mswms.mss_plot_driver import VerticalSectionDriver, HorizontalSectionDriver, LinearSectionDriver
import mss_wms_settings
import mslib.mswms.mpl_vsec_styles as mpl_vsec_styles
import mslib.mswms.mpl_hsec_styles as mpl_hsec_styles
import mslib.mswms.mpl_lsec_styles as mpl_lsec_styles
import mslib.mswms.gallery_builder


def is_image_transparent(img):
    with Image.open(io.BytesIO(img)) as image:
        if image.mode == "P":
            transparent = image.info.get("transparency", -1)
            for _, index in image.getcolors():
                if index == transparent:
                    return True
        elif image.mode == "RGBA":
            return image.getextrema()[3][0] < 255
        return False


class Test_VSec(object):
    def setup(self):
        p1 = [45.00, 8.]
        p2 = [50.00, 12.]
        p3 = [51.00, 15.]
        p4 = [48.00, 11.]
        data = mss_wms_settings.data["ecmwf_EUR_LL015"]
        data.setup()

        self.path = [p1, p2, p3, p4]
        self.bbox = [3, 500, 3, 10]
        self.init_time = datetime(2012, 10, 17, 12)
        self.valid_time = datetime(2012, 10, 17, 12)
        self.vsec = VerticalSectionDriver(data)

    def plot(self, plot_object, style="default", noframe=False, transparent=False, draw_verticals=False,
             return_format="image/png"):
        self.vsec.set_plot_parameters(plot_object=plot_object,
                                      bbox=self.bbox,
                                      vsec_path=self.path,
                                      vsec_numpoints=101,
                                      vsec_path_connection='greatcircle',
                                      init_time=self.init_time,
                                      valid_time=self.valid_time,
                                      style=style,
                                      noframe=noframe,
                                      draw_verticals=draw_verticals,
                                      transparent=transparent,
                                      return_format=return_format,
                                      show=False)
        return self.vsec.plot()

    def test_repeated_locations(self):
        p1 = [45.00, 8.]
        p2 = [50.00, 12.]
        self.path = [p1, p1]
        img = self.plot(mpl_vsec_styles.VS_TemperatureStyle_01(driver=self.vsec))
        assert img is not None
        self.path = [p1, p1, p2]
        img = self.plot(mpl_vsec_styles.VS_TemperatureStyle_01(driver=self.vsec))
        assert img is not None
        self.path = [p1, p2, p2]
        img = self.plot(mpl_vsec_styles.VS_TemperatureStyle_01(driver=self.vsec))
        assert img is not None

    def test_VS_verticals(self):
        img = self.plot(mpl_vsec_styles.VS_TemperatureStyle_01(driver=self.vsec), draw_verticals=True)
        assert img is not None
        assert not is_image_transparent(img)

    def test_VS_transparent(self):
        img = self.plot(mpl_vsec_styles.VS_TemperatureStyle_01(driver=self.vsec), transparent=True)
        assert img is not None
        assert is_image_transparent(img)

    def test_xml(self):
        img = self.plot(mpl_vsec_styles.VS_TemperatureStyle_01(driver=self.vsec), return_format="text/xml")
        assert img is not None
        ElementTree.fromstring(img)

    def test_VS_TemperatureStyle_01(self):
        img = self.plot(mpl_vsec_styles.VS_TemperatureStyle_01(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_TemperatureStyle_01(driver=self.vsec), noframe=True)
        assert noframe != img

    def test_VS_GenericStyle(self):
        img = self.plot(mpl_vsec_styles.VS_GenericStyle_PL_mole_fraction_of_ozone_in_air(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_GenericStyle_PL_mole_fraction_of_ozone_in_air(driver=self.vsec),
                            noframe=True)
        assert noframe != img

        img = self.plot(mpl_vsec_styles.VS_GenericStyle_TL_mole_fraction_of_ozone_in_air(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_GenericStyle_TL_mole_fraction_of_ozone_in_air(driver=self.vsec),
                            noframe=True)
        assert noframe != img

    def test_VS_CloudsStyle_01(self):
        img = self.plot(mpl_vsec_styles.VS_CloudsStyle_01(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_CloudsStyle_01(driver=self.vsec), noframe=True)
        assert noframe != img

    def test_VS_CloudsWindStyle_01(self):
        img = self.plot(mpl_vsec_styles.VS_CloudsWindStyle_01(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_CloudsWindStyle_01(driver=self.vsec), noframe=True)
        assert noframe != img

    def test_VS_RelativeHumdityStyle_01(self):
        img = self.plot(mpl_vsec_styles.VS_RelativeHumdityStyle_01(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_RelativeHumdityStyle_01(driver=self.vsec), noframe=True)
        assert noframe != img

    def test_VS_SpecificHumdityStyle_01(self):
        img = self.plot(mpl_vsec_styles.VS_SpecificHumdityStyle_01(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_SpecificHumdityStyle_01(driver=self.vsec), noframe=True)
        assert noframe != img

    def test_VS_VerticalVelocityStyle_01(self):
        img = self.plot(mpl_vsec_styles.VS_VerticalVelocityStyle_01(driver=self.vsec))
        assert img is not None

    def test_VS_HorizontalVelocityStyle_01(self):
        img = self.plot(mpl_vsec_styles.VS_HorizontalVelocityStyle_01(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_HorizontalVelocityStyle_01(driver=self.vsec), noframe=True)
        assert noframe != img

    def test_VS_PotentialVorticityStyle_01(self):
        img = self.plot(mpl_vsec_styles.VS_PotentialVorticityStyle_01(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_PotentialVorticityStyle_01(driver=self.vsec), noframe=True)
        assert noframe != img

    def test_VS_ProbabilityOfWCBStyle_01(self):
        img = self.plot(mpl_vsec_styles.VS_ProbabilityOfWCBStyle_01(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_ProbabilityOfWCBStyle_01(driver=self.vsec), noframe=True)
        assert noframe != img

    def test_VS_LagrantoTrajStyle_PL_01(self):
        pytest.skip("data not available")
        img = self.plot(mpl_vsec_styles.VS_LagrantoTrajStyle_PL_01(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_LagrantoTrajStyle_PL_01(driver=self.vsec), noframe=True)
        assert noframe != img

    def test_VS_EMACEyja_Style_01(self):
        img = self.plot(mpl_vsec_styles.VS_EMACEyja_Style_01(driver=self.vsec))
        assert img is not None
        noframe = self.plot(mpl_vsec_styles.VS_EMACEyja_Style_01(driver=self.vsec), noframe=True)
        assert noframe != img

    def test_VS_gallery_template(self):
        pytest.skip('Test can be biased. In pytest-reverse when there is not a plot_examples it can''t import')
        # ToDo Test Data have to be written to a random tmp dir and that may become purged afterwards
        templates_location = os.path.join(mslib.mswms.gallery_builder.DOCS_LOCATION, "plot_examples")
        sys.path.append(templates_location)
        from VS_template import VS_Template

        img = self.plot(VS_Template(driver=self.vsec))
        assert img is not None


class Test_LSec(object):
    def setup(self):
        p1 = [45.00, 8., 25000]
        p2 = [50.00, 12., 25000]
        p3 = [51.00, 15., 25000]
        p4 = [48.00, 11., 25000]
        data = mss_wms_settings.data["ecmwf_EUR_LL015"]
        data.setup()

        self.path = [p1, p2, p3, p4]
        self.bbox = [500]
        self.init_time = datetime(2012, 10, 17, 12)
        self.valid_time = datetime(2012, 10, 17, 12)
        self.lsec = LinearSectionDriver(data)

    def plot(self, plot_object):
        self.lsec.set_plot_parameters(plot_object=plot_object,
                                      bbox=self.bbox,
                                      lsec_path=self.path,
                                      lsec_numpoints=self.bbox[0],
                                      init_time=self.init_time,
                                      valid_time=self.valid_time)
        return self.lsec.plot()

    def test_repeated_locations(self):
        p1 = [45.00, 8., 25000]
        p2 = [50.00, 12., 25000]
        self.path = [p1, p1]
        img = self.plot(mpl_lsec_styles.LS_DefaultStyle(driver=self.lsec))
        assert img is not None
        self.path = [p1, p1, p2]
        img = self.plot(mpl_lsec_styles.LS_DefaultStyle(driver=self.lsec))
        assert img is not None
        self.path = [p1, p2, p2]
        img = self.plot(mpl_lsec_styles.LS_DefaultStyle(driver=self.lsec))
        assert img is not None

    def test_LS_DefaultStyle(self):
        for variable in ["air_temperature", "specific_humidity",
                         "cloud_area_fraction_in_atmosphere_layer", "specific_cloud_ice_water_content",
                         "specific_cloud_liquid_water_content", "ertel_potential_vorticity"]:
            img = self.plot(mpl_lsec_styles.LS_DefaultStyle(driver=self.lsec, variable=variable))
            assert img is not None

    def test_LS_DefaultStyle_PL(self):
        img = self.plot(mpl_lsec_styles.LS_DefaultStyle(driver=self.lsec, variable="air_potential_temperature",
                                                        filetype="pl"))
        assert img is not None

    def test_LS_RelativeHumdityStyle_01(self):
        img = self.plot(mpl_lsec_styles.LS_RelativeHumdityStyle_01(driver=self.lsec))
        assert img is not None

    def test_LS_HorizontalVelocityStyle_01(self):
        img = self.plot(mpl_lsec_styles.LS_HorizontalVelocityStyle_01(driver=self.lsec))
        assert img is not None

    def test_LS_VerticalVelocityStyle_01(self):
        img = self.plot(mpl_lsec_styles.LS_VerticalVelocityStyle_01(driver=self.lsec))
        assert img is not None


class Test_HSec(object):
    def setup(self):
        data = mss_wms_settings.data["ecmwf_EUR_LL015"]
        data.setup()

        self.bbox = [-22.5, 27.5, 55, 62.5]

        self.init_time = datetime(2012, 10, 17, 12)
        self.valid_time = datetime(2012, 10, 17, 12)
        self.hsec = HorizontalSectionDriver(data)

    def plot(self, plot_object, style="default", level=None, crs="EPSG:4326", bbox=None, noframe=False,
             transparent=False):
        if bbox is None:
            bbox = self.bbox
        self.hsec.set_plot_parameters(plot_object=plot_object, bbox=bbox, level=level, crs=crs,
                                      init_time=self.init_time, valid_time=self.valid_time, style=style,
                                      noframe=noframe, show=False, transparent=transparent)
        return self.hsec.plot()

    @pytest.mark.parametrize("crs", [
        "EPSG:4326",
        "EPSG:77890010", "EPSG:77790010",
        "MSS:stere,20,40,40", "MSS:lcc,20,0,40,20", "MSS:cass,20,40", "MSS:merc,40"])
    def test_degree_crs_codes(self, crs):
        img = self.plot(mpl_hsec_styles.HS_MSLPStyle_01(driver=self.hsec), crs=crs)
        assert img is not None

    @pytest.mark.parametrize("crs", ["EPSG:3031", "EPSG:3857", "EPSG:3413", "EPSG:3995"])
    def test_meter_crs_codes(self, crs):
        bbox_meter = [-1e7, -1e7, 1e7, 1e7]
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")  # disable warnings as there is no data for the cutout
            img = self.plot(mpl_hsec_styles.HS_MSLPStyle_01(driver=self.hsec), crs=crs, bbox=bbox_meter)
        assert img is not None

    @pytest.mark.parametrize("crs", ["EPSG:12345678", "FNORD", "MSS:lagranto"])
    def test_invalid_crs_codes(self, crs):
        with pytest.raises(ValueError):
            self.plot(mpl_hsec_styles.HS_MSLPStyle_01(driver=self.hsec), crs=crs)

    def test_repeated_locations(self):
        p1 = [45.00, 8.]
        p2 = [50.00, 12.]
        self.path = [p1, p1]
        img = self.plot(mpl_hsec_styles.HS_TemperatureStyle_ML_01(driver=self.hsec), level=10)
        assert img is not None
        self.path = [p1, p1, p2]
        img = self.plot(mpl_hsec_styles.HS_TemperatureStyle_ML_01(driver=self.hsec), level=10)
        assert img is not None
        self.path = [p1, p2, p2]
        img = self.plot(mpl_hsec_styles.HS_TemperatureStyle_ML_01(driver=self.hsec), level=10)
        assert img is not None

    def test_HS_CloudsStyle_01(self):
        for style in ["TOT", "HIGH", "MED", "LOW"]:
            img = self.plot(mpl_hsec_styles.HS_CloudsStyle_01(driver=self.hsec), style=style)
            assert img is not None
            noframe = self.plot(mpl_hsec_styles.HS_CloudsStyle_01(driver=self.hsec), style=style, noframe=True)
            assert noframe is not None
            assert noframe != img
            assert not is_image_transparent(img)

    def test_HS_transparent(self):
        img = self.plot(mpl_hsec_styles.HS_MSLPStyle_01(driver=self.hsec), transparent=True)
        assert img is not None
        assert is_image_transparent(img)

    def test_HS_MSLPStyle_01(self):
        img = self.plot(mpl_hsec_styles.HS_MSLPStyle_01(driver=self.hsec))
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_MSLPStyle_01(driver=self.hsec), noframe=True)
        assert noframe is not None
        assert noframe != img

    def test_HS_SEAStyle_01(self):
        img = self.plot(mpl_hsec_styles.HS_SEAStyle_01(driver=self.hsec))
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_SEAStyle_01(driver=self.hsec), noframe=True)
        assert noframe is not None
        assert noframe != img

    @pytest.mark.parametrize("style", ["PCOL", "CONT"])
    def test_HS_SeaIceStyle_01(self, style):
        img = self.plot(mpl_hsec_styles.HS_SeaIceStyle_01(driver=self.hsec), style=style)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_SeaIceStyle_01(driver=self.hsec), style=style, noframe=True)
        assert noframe is not None
        assert noframe != img

    def test_HS_TemperatureStyle_ML_01(self):
        img = self.plot(mpl_hsec_styles.HS_TemperatureStyle_ML_01(driver=self.hsec), level=10)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_TemperatureStyle_ML_01(driver=self.hsec), level=10, noframe=True)
        assert noframe is not None
        assert noframe != img

    def test_HS_TemperatureStyle_PL_01(self):
        img = self.plot(mpl_hsec_styles.HS_TemperatureStyle_PL_01(driver=self.hsec), level=800)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_TemperatureStyle_PL_01(driver=self.hsec), level=800, noframe=True)
        assert noframe is not None
        assert noframe != img

    def test_HS_GeopotentialWindStyle_PL(self):
        img = self.plot(mpl_hsec_styles.HS_GeopotentialWindStyle_PL(driver=self.hsec), level=300)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_GeopotentialWindStyle_PL(driver=self.hsec), level=300, noframe=True)
        assert noframe is not None
        assert noframe != img

    @pytest.mark.parametrize("style", ["default", "nonlinear", "auto", "log", "autolog"])
    def test_HS_GenericStyle_styles(self, style):
        img = self.plot(
            mpl_hsec_styles.HS_GenericStyle_PL_mole_fraction_of_ozone_in_air(driver=self.hsec),
            level=300, style=style)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_GenericStyle_PL_mole_fraction_of_ozone_in_air(driver=self.hsec),
                            level=300, style=style, noframe=True)
        assert noframe is not None
        assert noframe != img

    def test_HS_GenericStyle_other(self):
        img = self.plot(mpl_hsec_styles.HS_GenericStyle_TL_mole_fraction_of_ozone_in_air(driver=self.hsec), level=300)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_GenericStyle_TL_mole_fraction_of_ozone_in_air(driver=self.hsec),
                            level=300, noframe=True)
        assert noframe != img

        img = self.plot(
            mpl_hsec_styles.HS_GenericStyle_PL_ertel_potential_vorticity(driver=self.hsec),
            style="ertel_potential_vorticity", level=300)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_GenericStyle_PL_ertel_potential_vorticity(driver=self.hsec),
                            level=300, style="ertel_potential_vorticity", noframe=True)
        assert noframe is not None
        assert noframe != img

        img = self.plot(
            mpl_hsec_styles.HS_GenericStyle_PL_equivalent_latitude(driver=self.hsec),
            style="equivalent_latitude", level=300)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_GenericStyle_PL_equivalent_latitude(driver=self.hsec),
                            level=300, style="equivalent_latitude", noframe=True)
        assert noframe is not None
        assert noframe != img

    def test_HS_RelativeHumidityStyle_PL_01(self):
        img = self.plot(mpl_hsec_styles.HS_RelativeHumidityStyle_PL_01(driver=self.hsec), level=300)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_RelativeHumidityStyle_PL_01(driver=self.hsec), level=300, noframe=True)
        assert noframe is not None
        assert noframe != img

    def test_HS_EQPTStyle_PL_01(self):
        img = self.plot(mpl_hsec_styles.HS_EQPTStyle_PL_01(driver=self.hsec), level=300)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_EQPTStyle_PL_01(driver=self.hsec), level=300, noframe=True)
        assert noframe != img

    def test_HS_WStyle_PL_01(self):
        img = self.plot(mpl_hsec_styles.HS_WStyle_PL_01(driver=self.hsec), level=300)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_WStyle_PL_01(driver=self.hsec), level=300, noframe=True)
        assert noframe != img

    def test_HS_DivStyle_PL_01(self):
        img = self.plot(mpl_hsec_styles.HS_DivStyle_PL_01(driver=self.hsec), level=300)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_DivStyle_PL_01(driver=self.hsec), level=300, noframe=True)
        assert noframe != img

    def test_HS_EMAC_TracerStyle_ML_01(self):
        img = self.plot(mpl_hsec_styles.HS_EMAC_TracerStyle_ML_01(driver=self.hsec), level=10)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_EMAC_TracerStyle_ML_01(driver=self.hsec), level=10, noframe=True)
        assert noframe != img

    def test_HS_EMAC_TracerStyle_SFC_01(self):
        img = self.plot(mpl_hsec_styles.HS_EMAC_TracerStyle_SFC_01(driver=self.hsec))
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_EMAC_TracerStyle_SFC_01(driver=self.hsec), noframe=True)
        assert noframe != img

    def test_HS_PVTropoStyle_PV_01(self):
        # test fractional levels and non-existing levels
        for style in ["PRES", "PT", "GEOP"]:
            img = self.plot(mpl_hsec_styles.HS_PVTropoStyle_PV_01(driver=self.hsec), level=2.5, style=style)
            assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_PVTropoStyle_PV_01(driver=self.hsec), level=2.5, style=style,
                            noframe=True)
        assert noframe != img
        with pytest.raises(ValueError):
            self.plot(mpl_hsec_styles.HS_PVTropoStyle_PV_01(driver=self.hsec), level=2.75)

    def test_HS_VIProbWCB_Style_01(self):
        img = self.plot(mpl_hsec_styles.HS_VIProbWCB_Style_01(driver=self.hsec))
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_VIProbWCB_Style_01(driver=self.hsec), noframe=True)
        assert noframe != img

    def test_HS_LagrantoTrajStyle_PL_01(self):
        img = self.plot(mpl_hsec_styles.HS_LagrantoTrajStyle_PL_01(driver=self.hsec), level=300)
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_LagrantoTrajStyle_PL_01(driver=self.hsec), level=300, noframe=True)
        assert noframe != img

    def test_HS_BLH_MSLP_Style_01(self):
        img = self.plot(mpl_hsec_styles.HS_BLH_MSLP_Style_01(driver=self.hsec))
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_BLH_MSLP_Style_01(driver=self.hsec), noframe=True)
        assert noframe != img

    def test_HS_Meteosat_BT108_01(self):
        img = self.plot(mpl_hsec_styles.HS_Meteosat_BT108_01(driver=self.hsec))
        assert img is not None
        noframe = self.plot(mpl_hsec_styles.HS_Meteosat_BT108_01(driver=self.hsec), noframe=True)
        assert noframe != img

    def test_HS_gallery_template(self):
        pytest.skip('Test can be biased. In pytest-reverse when there is not a plot_examples it can''t import')
        # ToDo Test Data have to be written to a random tmp dir and that may become purged afterwards
        templates_location = os.path.join(mslib.mswms.gallery_builder.DOCS_LOCATION, "plot_examples")
        sys.path.append(templates_location)
        from HS_template import HS_Template

        img = self.plot(HS_Template(driver=self.hsec), level=300)
        assert img is not None



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_mscolab_operation_window.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MscolabOperation(object):
    def setupUi(self, MscolabOperation):
        MscolabOperation.setObjectName("MscolabOperation")
        MscolabOperation.setWindowModality(QtCore.Qt.NonModal)
        MscolabOperation.setEnabled(True)
        MscolabOperation.resize(867, 687)
        MscolabOperation.setMinimumSize(QtCore.QSize(600, 400))
        self.centralwidget = QtWidgets.QWidget(MscolabOperation)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.user_info = QtWidgets.QLabel(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.user_info.sizePolicy().hasHeightForWidth())
        self.user_info.setSizePolicy(sizePolicy)
        self.user_info.setObjectName("user_info")
        self.horizontalLayout_3.addWidget(self.user_info)
        self.proj_info = QtWidgets.QLabel(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.proj_info.sizePolicy().hasHeightForWidth())
        self.proj_info.setSizePolicy(sizePolicy)
        self.proj_info.setObjectName("proj_info")
        self.horizontalLayout_3.addWidget(self.proj_info)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.horizontalLayout_3.setStretch(0, 1)
        self.horizontalLayout_3.setStretch(1, 1)
        self.horizontalLayout_3.setStretch(2, 2)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.collaboratorsList = QtWidgets.QListWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.collaboratorsList.sizePolicy().hasHeightForWidth())
        self.collaboratorsList.setSizePolicy(sizePolicy)
        self.collaboratorsList.setMinimumSize(QtCore.QSize(256, 300))
        self.collaboratorsList.setObjectName("collaboratorsList")
        self.horizontalLayout_4.addWidget(self.collaboratorsList)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_5.setSpacing(2)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.searchMessageLineEdit = QtWidgets.QLineEdit(self.centralwidget)
        self.searchMessageLineEdit.setObjectName("searchMessageLineEdit")
        self.horizontalLayout_5.addWidget(self.searchMessageLineEdit)
        self.searchPrevBtn = QtWidgets.QPushButton(self.centralwidget)
        self.searchPrevBtn.setObjectName("searchPrevBtn")
        self.horizontalLayout_5.addWidget(self.searchPrevBtn)
        self.searchNextBtn = QtWidgets.QPushButton(self.centralwidget)
        self.searchNextBtn.setObjectName("searchNextBtn")
        self.horizontalLayout_5.addWidget(self.searchNextBtn)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)
        self.messageList = QtWidgets.QListWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.messageList.sizePolicy().hasHeightForWidth())
        self.messageList.setSizePolicy(sizePolicy)
        self.messageList.setMinimumSize(QtCore.QSize(539, 300))
        self.messageList.setObjectName("messageList")
        self.verticalLayout_3.addWidget(self.messageList)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.previewBtn = QtWidgets.QPushButton(self.centralwidget)
        self.previewBtn.setToolTip("")
        self.previewBtn.setAutoDefault(False)
        self.previewBtn.setDefault(False)
        self.previewBtn.setFlat(False)
        self.previewBtn.setObjectName("previewBtn")
        self.horizontalLayout_2.addWidget(self.previewBtn)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.messageTextContainer = QtWidgets.QWidget(self.centralwidget)
        self.messageTextContainer.setObjectName("messageTextContainer")
        self.horizontalLayout.addWidget(self.messageTextContainer)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.sendMessageBtn = QtWidgets.QPushButton(self.centralwidget)
        self.sendMessageBtn.setDefault(True)
        self.sendMessageBtn.setObjectName("sendMessageBtn")
        self.verticalLayout.addWidget(self.sendMessageBtn)
        self.uploadBtn = QtWidgets.QPushButton(self.centralwidget)
        self.uploadBtn.setObjectName("uploadBtn")
        self.verticalLayout.addWidget(self.uploadBtn)
        self.editMessageBtn = QtWidgets.QPushButton(self.centralwidget)
        self.editMessageBtn.setToolTip("")
        self.editMessageBtn.setObjectName("editMessageBtn")
        self.verticalLayout.addWidget(self.editMessageBtn)
        self.cancelBtn = QtWidgets.QPushButton(self.centralwidget)
        self.cancelBtn.setEnabled(True)
        self.cancelBtn.setObjectName("cancelBtn")
        self.verticalLayout.addWidget(self.cancelBtn)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.horizontalLayout.setStretch(0, 2)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.verticalLayout_3.addLayout(self.verticalLayout_2)
        self.verticalLayout_3.setStretch(1, 4)
        self.verticalLayout_3.setStretch(2, 1)
        self.horizontalLayout_4.addLayout(self.verticalLayout_3)
        self.horizontalLayout_4.setStretch(1, 1)
        self.verticalLayout_4.addLayout(self.horizontalLayout_4)
        self.gridLayout.addLayout(self.verticalLayout_4, 0, 0, 1, 1)
        MscolabOperation.setCentralWidget(self.centralwidget)
        self.actionCloseWindow = QtWidgets.QAction(MscolabOperation)
        self.actionCloseWindow.setObjectName("actionCloseWindow")
        MscolabOperation.addAction(self.actionCloseWindow)

        self.retranslateUi(MscolabOperation)
        self.actionCloseWindow.triggered.connect(MscolabOperation.close)
        QtCore.QMetaObject.connectSlotsByName(MscolabOperation)

    def retranslateUi(self, MscolabOperation):
        _translate = QtCore.QCoreApplication.translate
        MscolabOperation.setWindowTitle(_translate("MscolabOperation", "Mscolab Operation Chat"))
        self.user_info.setText(_translate("MscolabOperation", "Logged In: "))
        self.proj_info.setText(_translate("MscolabOperation", "Operation:"))
        self.searchMessageLineEdit.setPlaceholderText(_translate("MscolabOperation", "Search Message"))
        self.searchPrevBtn.setText(_translate("MscolabOperation", "Previous"))
        self.searchNextBtn.setText(_translate("MscolabOperation", "Next"))
        self.previewBtn.setText(_translate("MscolabOperation", "Preview"))
        self.sendMessageBtn.setText(_translate("MscolabOperation", "send"))
        self.uploadBtn.setToolTip(_translate("MscolabOperation", "Upload image or file"))
        self.uploadBtn.setText(_translate("MscolabOperation", "Upload"))
        self.editMessageBtn.setText(_translate("MscolabOperation", "Update"))
        self.cancelBtn.setText(_translate("MscolabOperation", "Cancel"))
        self.actionCloseWindow.setText(_translate("MscolabOperation", "CloseWindow"))
        self.actionCloseWindow.setShortcut(_translate("MscolabOperation", "Ctrl+W"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_mscolab_help_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_mscolabHelpDialog(object):
    def setupUi(self, mscolabHelpDialog):
        mscolabHelpDialog.setObjectName("mscolabHelpDialog")
        mscolabHelpDialog.resize(710, 703)
        self.gridLayout = QtWidgets.QGridLayout(mscolabHelpDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(mscolabHelpDialog)
        self.tabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.tabWidget.setObjectName("tabWidget")
        self.connectingTab = QtWidgets.QWidget()
        self.connectingTab.setObjectName("connectingTab")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.connectingTab)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setContentsMargins(-1, 10, -1, 10)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem)
        self.label_9 = QtWidgets.QLabel(self.connectingTab)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_8.addWidget(self.label_9)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem1)
        self.verticalLayout_3.addLayout(self.horizontalLayout_8)
        self.textEdit_17 = QtWidgets.QTextEdit(self.connectingTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textEdit_17.sizePolicy().hasHeightForWidth())
        self.textEdit_17.setSizePolicy(sizePolicy)
        self.textEdit_17.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textEdit_17.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textEdit_17.setReadOnly(True)
        self.textEdit_17.setObjectName("textEdit_17")
        self.verticalLayout_3.addWidget(self.textEdit_17)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem2)
        self.label_10 = QtWidgets.QLabel(self.connectingTab)
        self.label_10.setText("")
        self.label_10.setPixmap(QtGui.QPixmap(":/images/mscolab-help/connect.png"))
        self.label_10.setScaledContents(True)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_9.addWidget(self.label_10)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem3)
        self.verticalLayout_3.addLayout(self.horizontalLayout_9)
        self.textEdit_18 = QtWidgets.QTextEdit(self.connectingTab)
        self.textEdit_18.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textEdit_18.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textEdit_18.setReadOnly(True)
        self.textEdit_18.setObjectName("textEdit_18")
        self.verticalLayout_3.addWidget(self.textEdit_18)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setContentsMargins(-1, -1, -1, 8)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem4)
        self.label_12 = QtWidgets.QLabel(self.connectingTab)
        self.label_12.setText("")
        self.label_12.setPixmap(QtGui.QPixmap(":/images/mscolab-help/add-user.png"))
        self.label_12.setScaledContents(True)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_10.addWidget(self.label_12)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem5)
        self.verticalLayout_3.addLayout(self.horizontalLayout_10)
        self.verticalLayout_3.setStretch(1, 1)
        self.verticalLayout_3.setStretch(3, 1)
        self.gridLayout_4.addLayout(self.verticalLayout_3, 0, 0, 1, 1)
        self.tabWidget.addTab(self.connectingTab, "")
        self.createOperationTab = QtWidgets.QWidget()
        self.createOperationTab.setObjectName("createOperationTab")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.createOperationTab)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.textEdit_20 = QtWidgets.QTextEdit(self.createOperationTab)
        self.textEdit_20.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.textEdit_20.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textEdit_20.setReadOnly(True)
        self.textEdit_20.setObjectName("textEdit_20")
        self.verticalLayout_4.addWidget(self.textEdit_20)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(spacerItem6)
        self.label_13 = QtWidgets.QLabel(self.createOperationTab)
        self.label_13.setText("")
        self.label_13.setPixmap(QtGui.QPixmap(":/images/mscolab-help/add-operation.png"))
        self.label_13.setScaledContents(True)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_11.addWidget(self.label_13)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(spacerItem7)
        self.verticalLayout_4.addLayout(self.horizontalLayout_11)
        self.gridLayout_3.addLayout(self.verticalLayout_4, 0, 0, 1, 1)
        self.tabWidget.addTab(self.createOperationTab, "")
        self.adminTab = QtWidgets.QWidget()
        self.adminTab.setObjectName("adminTab")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.adminTab)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.textEdit_22 = QtWidgets.QTextEdit(self.adminTab)
        self.textEdit_22.setEnabled(True)
        self.textEdit_22.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.textEdit_22.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textEdit_22.setUndoRedoEnabled(False)
        self.textEdit_22.setReadOnly(True)
        self.textEdit_22.setObjectName("textEdit_22")
        self.verticalLayout_5.addWidget(self.textEdit_22)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem8)
        self.label_14 = QtWidgets.QLabel(self.adminTab)
        self.label_14.setText("")
        self.label_14.setPixmap(QtGui.QPixmap(":/images/mscolab-help/admin-window.png"))
        self.label_14.setScaledContents(True)
        self.label_14.setObjectName("label_14")
        self.horizontalLayout_12.addWidget(self.label_14)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem9)
        self.verticalLayout_5.addLayout(self.horizontalLayout_12)
        self.gridLayout_5.addLayout(self.verticalLayout_5, 0, 0, 1, 1)
        self.tabWidget.addTab(self.adminTab, "")
        self.chatTab = QtWidgets.QWidget()
        self.chatTab.setObjectName("chatTab")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.chatTab)
        self.gridLayout_7.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.textEdit_23 = QtWidgets.QTextEdit(self.chatTab)
        self.textEdit_23.setEnabled(True)
        self.textEdit_23.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.textEdit_23.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textEdit_23.setUndoRedoEnabled(False)
        self.textEdit_23.setReadOnly(True)
        self.textEdit_23.setObjectName("textEdit_23")
        self.verticalLayout_6.addWidget(self.textEdit_23)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        spacerItem10 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem10)
        self.label = QtWidgets.QLabel(self.chatTab)
        self.label.setText("")
        self.label.setPixmap(QtGui.QPixmap(":/images/mscolab-help/chat-window.png"))
        self.label.setScaledContents(True)
        self.label.setObjectName("label")
        self.horizontalLayout_13.addWidget(self.label)
        spacerItem11 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem11)
        self.verticalLayout_6.addLayout(self.horizontalLayout_13)
        self.gridLayout_7.addLayout(self.verticalLayout_6, 0, 0, 1, 1)
        self.tabWidget.addTab(self.chatTab, "")
        self.versionTab = QtWidgets.QWidget()
        self.versionTab.setObjectName("versionTab")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.versionTab)
        self.gridLayout_9.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.textEdit_24 = QtWidgets.QTextEdit(self.versionTab)
        self.textEdit_24.setEnabled(True)
        self.textEdit_24.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textEdit_24.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textEdit_24.setUndoRedoEnabled(False)
        self.textEdit_24.setReadOnly(True)
        self.textEdit_24.setObjectName("textEdit_24")
        self.verticalLayout_7.addWidget(self.textEdit_24)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        spacerItem12 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem12)
        self.label_16 = QtWidgets.QLabel(self.versionTab)
        self.label_16.setText("")
        self.label_16.setPixmap(QtGui.QPixmap(":/images/mscolab-help/version-window.png"))
        self.label_16.setScaledContents(True)
        self.label_16.setObjectName("label_16")
        self.horizontalLayout_14.addWidget(self.label_16)
        spacerItem13 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem13)
        self.verticalLayout_7.addLayout(self.horizontalLayout_14)
        self.gridLayout_9.addLayout(self.verticalLayout_7, 0, 0, 1, 1)
        self.tabWidget.addTab(self.versionTab, "")
        self.operationWorkTab = QtWidgets.QWidget()
        self.operationWorkTab.setObjectName("operationWorkTab")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.operationWorkTab)
        self.gridLayout_8.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.textEdit_25 = QtWidgets.QTextEdit(self.operationWorkTab)
        self.textEdit_25.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.textEdit_25.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textEdit_25.setReadOnly(True)
        self.textEdit_25.setObjectName("textEdit_25")
        self.verticalLayout_8.addWidget(self.textEdit_25)
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        spacerItem14 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_15.addItem(spacerItem14)
        self.label_17 = QtWidgets.QLabel(self.operationWorkTab)
        self.label_17.setText("")
        self.label_17.setPixmap(QtGui.QPixmap(":/images/mscolab-help/merge-window.png"))
        self.label_17.setScaledContents(True)
        self.label_17.setObjectName("label_17")
        self.horizontalLayout_15.addWidget(self.label_17)
        spacerItem15 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_15.addItem(spacerItem15)
        self.verticalLayout_8.addLayout(self.horizontalLayout_15)
        self.gridLayout_8.addLayout(self.verticalLayout_8, 0, 0, 1, 1)
        self.tabWidget.addTab(self.operationWorkTab, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem16 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem16)
        self.okayBtn = QtWidgets.QPushButton(mscolabHelpDialog)
        self.okayBtn.setObjectName("okayBtn")
        self.horizontalLayout.addWidget(self.okayBtn)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.retranslateUi(mscolabHelpDialog)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(mscolabHelpDialog)

    def retranslateUi(self, mscolabHelpDialog):
        _translate = QtCore.QCoreApplication.translate
        mscolabHelpDialog.setWindowTitle(_translate("mscolabHelpDialog", "Help"))
        self.label_9.setText(_translate("mscolabHelpDialog", "Welcome to Mission Support Collaboration!"))
        self.textEdit_17.setHtml(_translate("mscolabHelpDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'.AppleSystemUIFont\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">We suggest you go through this guide before you start using mscolab to understand all the features it has to offer.<br /><br />1. To get started, click on the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">Connect to MSColab</span><span style=\" font-family:\'.SF NS Text\';\"> button on the main window.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">2. In the connect window that appears, enter your MSColab Server URL and press the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">Connect</span><span style=\" font-family:\'.SF NS Text\';\"> button.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p></body></html>"))
        self.textEdit_18.setHtml(_translate("mscolabHelpDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'.AppleSystemUIFont\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">If you\'re using MSColab for the first time, click on the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Add User&quot;</span><span style=\" font-family:\'.SF NS Text\';\"> button after connecting and register yourself by filling the required details. </span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">The status bar will notify if/not the new user is registered on the server. If successfully registered, you\'ll be taken to the login page where you can login with the new user credentials.</span></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.connectingTab), _translate("mscolabHelpDialog", "Connecting"))
        self.textEdit_20.setHtml(_translate("mscolabHelpDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'.AppleSystemUIFont\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">1. Each flight track in mscolab is called a </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Operation&quot;</span><span style=\" font-family:\'.SF NS Text\';\">. You can create your operation by clicking on the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;MSColab Operation&quot;</span><span style=\" font-family:\'.SF NS Text\';\"> menu under the &quot;New&quot; section of File menu and filling in the required details.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">2. The operation list shows all the operations you have access to along with your access level to the operation. There are </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">4 access levels</span><span style=\" font-family:\'.SF NS Text\';\"> you can have to a operation. These are: </span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">    1. Creator</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">    2. Admin</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">    3. Collaborator</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">    4. Viewer</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">3. You can select a operation by </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">double-clicking</span><span style=\" font-family:\'.SF NS Text\';\"> on it in the operation list and then start working on it.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">4. Select a operation and you can start editing the flight track in 4 different views - Top, Side and Table, Linear. (Note: Already open views will automatically change the waypoints model to the newly selected operation\'s waypoints model)</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">5. You can use the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">Import/Export</span><span style=\" font-family:\'.SF NS Text\';\"> menu from the &quot;File&quot; menu to import or export Flighttrack files in FTML/CSV or any other supported plugin format.</span></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.createOperationTab), _translate("mscolabHelpDialog", "Creating a Operation"))
        self.textEdit_22.setHtml(_translate("mscolabHelpDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'.AppleSystemUIFont\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">To collaborate with other users, you can add them to a operation if you are the creator of the operation or have administrator access. Click on the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Manage Users&quot;</span><span style=\" font-family:\'.SF NS Text\';\"> sub-menu under the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">Operation menu</span><span style=\" font-family:\'.SF NS Text\';\">, after selecting the operation.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">A user can have 4 access levels:</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">1. </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">Creator</span><span style=\" font-family:\'.SF NS Text\';\"> - The user that creates the operation. This user\'s permission can\'t be changed. This user has all the access an Admin has and can also delete the operation.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">2. </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">Admin</span><span style=\" font-family:\'.SF NS Text\';\"> - The admin has permission to add and update access levels of other users working on the operation. They can also view the version history of the operation and checkout to a previous version.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">3. </span><span style=\" font-weight:600;\">Collaborator</span> - The collaborator can edit the flight track path and use the version history but has no ability to manage users.</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">4. </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">Viewer</span><span style=\" font-family:\'.SF NS Text\';\"> - The viewer has the least access. They can only view the flight tracks and can\'t edit anything.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">What can you do with the Mscolab Admin Window?</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">1. Select users on your Mscolab server and add them to your operation or update the access level of existing users.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">2. Clone the permissions from a different operation using the clone button at the top, allowing your team to quickly get started on a new operation.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">3. Easily search users using the search bar. You can use the username or email id to search for a user.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">4. Apply filter based on access level to find exactly who you need.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.adminTab), _translate("mscolabHelpDialog", "Adding Users to Operation"))
        self.textEdit_23.setHtml(_translate("mscolabHelpDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'.AppleSystemUIFont\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">You can chat with all the users added to a operation by opening the chat window using the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Chat&quot;</span><span style=\" font-family:\'.SF NS Text\';\"> sub-menu under the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">Operation menu</span><span style=\" font-family:\'.SF NS Text\';\">. You can send normal messages and upload images and documents as well. </span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">You can also use common markdown syntax to format your messages.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">1. *text* for italics</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">2. **text** for bold</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">3. - for bullet list</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">4. # for heading</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">5. [text](url) for adding link to text</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">You can use the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Preview&quot;</span><span style=\" font-family:\'.SF NS Text\';\"> button to preview how your message looks before sending it.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">There are other features supported like editing, deleting and replying to a message. Right click on any message to see these options.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.chatTab), _translate("mscolabHelpDialog", "Chat"))
        self.textEdit_24.setHtml(_translate("mscolabHelpDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'.AppleSystemUIFont\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">You can keep track of all the changes made to your operation by opening the version history window using the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Version History&quot;</span><span style=\" font-family:\'.SF NS Text\';\"> sub-menu under the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">Operation menu</span><span style=\" font-family:\'.SF NS Text\';\">.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">Every change made by any user working the operation is saved in the version history window. At any time you can revert to a preview version by just selecting the version from the list and clicking on the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Checkout&quot; </span><span style=\" font-family:\'.SF NS Text\';\">button.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">You can also give names to the important versions so you can view only the important named versions by changing the filter above the version list</span></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.versionTab), _translate("mscolabHelpDialog", "Manage Versions"))
        self.textEdit_25.setHtml(_translate("mscolabHelpDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'.AppleSystemUIFont\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">While working on a operation you can be working in 2 modes:</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">1. The </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">default mode</span><span style=\" font-family:\'.SF NS Text\';\"> is when the  </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Work Asynchronously&quot; </span><span style=\" font-family:\'.SF NS Text\';\">checkbox is </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">unchecked</span><span style=\" font-family:\'.SF NS Text\';\">. In this mode all your changes will be shared with all other users in real-time. Each change made will also be recorded in the version history window in this mode.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">2. If you just want to try some changes on your own and not share with everyone else, you can turn the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Work Asynchronously&quot; </span><span style=\" font-family:\'.SF NS Text\';\">checkbox </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">on</span><span style=\" font-family:\'.SF NS Text\';\">. In work locally mode your changes are kept on your machine and not shared with anyone. Once you\'re done making changes you can use the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Save to Server&quot;</span><span style=\" font-family:\'.SF NS Text\';\"> option in the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">Server Options drop-down list</span><span style=\" font-family:\'.SF NS Text\';\"> to save your changes with everyone. On clicking the save option a merge dialog is opened. Here you can choose to completely overwrite the shared flight track with your local one or choose what you want to keep from your local data and the shared data. You can also use the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Fetch from Server&quot; </span><span style=\" font-family:\'.SF NS Text\';\">option so you can merge the shared flight track data with your local data.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\';\">You can toggle between these modes at any moment you like. All your local changes are saved on your machine and are </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">not</span><span style=\" font-family:\'.SF NS Text\';\"> discarded. You can uncheck the </span><span style=\" font-family:\'.SF NS Text\'; font-weight:600;\">&quot;Work Asynchronously&quot;</span><span style=\" font-family:\'.SF NS Text\';\"> checkbox at any time, work on the shared file and then turn it back on and continue from where you left off.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.SF NS Text\';\"><br /></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.operationWorkTab), _translate("mscolabHelpDialog", "Working on a Operation"))
        self.okayBtn.setText(_translate("mscolabHelpDialog", "Okay"))

from . import resources_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_about_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AboutMSUIDialog(object):
    def setupUi(self, AboutMSUIDialog):
        AboutMSUIDialog.setObjectName("AboutMSUIDialog")
        AboutMSUIDialog.resize(1104, 600)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(AboutMSUIDialog.sizePolicy().hasHeightForWidth())
        AboutMSUIDialog.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(AboutMSUIDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.lblDummy = QtWidgets.QLabel(AboutMSUIDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblDummy.sizePolicy().hasHeightForWidth())
        self.lblDummy.setSizePolicy(sizePolicy)
        self.lblDummy.setMinimumSize(QtCore.QSize(100, 40))
        self.lblDummy.setMaximumSize(QtCore.QSize(100, 40))
        self.lblDummy.setText("")
        self.lblDummy.setObjectName("lblDummy")
        self.horizontalLayout_3.addWidget(self.lblDummy)
        self.lblName = QtWidgets.QLabel(AboutMSUIDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblName.sizePolicy().hasHeightForWidth())
        self.lblName.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(18)
        font.setBold(True)
        font.setWeight(75)
        self.lblName.setFont(font)
        self.lblName.setTextFormat(QtCore.Qt.PlainText)
        self.lblName.setAlignment(QtCore.Qt.AlignCenter)
        self.lblName.setObjectName("lblName")
        self.horizontalLayout_3.addWidget(self.lblName)
        self.lblPython = QtWidgets.QLabel(AboutMSUIDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblPython.sizePolicy().hasHeightForWidth())
        self.lblPython.setSizePolicy(sizePolicy)
        self.lblPython.setMinimumSize(QtCore.QSize(100, 40))
        self.lblPython.setMaximumSize(QtCore.QSize(100, 40))
        self.lblPython.setObjectName("lblPython")
        self.horizontalLayout_3.addWidget(self.lblPython)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        spacerItem = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        self.verticalLayout_2.addItem(spacerItem)
        self.textBrowser = QtWidgets.QTextBrowser(AboutMSUIDialog)
        self.textBrowser.setEnabled(True)
        self.textBrowser.setMinimumSize(QtCore.QSize(0, 200))
        self.textBrowser.setMaximumSize(QtCore.QSize(1310, 300))
        self.textBrowser.setObjectName("textBrowser")
        self.verticalLayout_2.addWidget(self.textBrowser)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lblVersion = QtWidgets.QLabel(AboutMSUIDialog)
        self.lblVersion.setObjectName("lblVersion")
        self.horizontalLayout_2.addWidget(self.lblVersion)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.lblChanges = QtWidgets.QLabel(AboutMSUIDialog)
        self.lblChanges.setOpenExternalLinks(True)
        self.lblChanges.setObjectName("lblChanges")
        self.horizontalLayout_2.addWidget(self.lblChanges)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        spacerItem2 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        self.verticalLayout_2.addItem(spacerItem2)
        self.lblLicense = QtWidgets.QLabel(AboutMSUIDialog)
        self.lblLicense.setObjectName("lblLicense")
        self.verticalLayout_2.addWidget(self.lblLicense)
        spacerItem3 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        self.verticalLayout_2.addItem(spacerItem3)
        self.lblCopyright = QtWidgets.QLabel(AboutMSUIDialog)
        self.lblCopyright.setObjectName("lblCopyright")
        self.verticalLayout_2.addWidget(self.lblCopyright)
        spacerItem4 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        self.verticalLayout_2.addItem(spacerItem4)
        self.lblLinks = QtWidgets.QLabel(AboutMSUIDialog)
        self.lblLinks.setOpenExternalLinks(True)
        self.lblLinks.setObjectName("lblLinks")
        self.verticalLayout_2.addWidget(self.lblLinks)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem5)
        self.btOK = QtWidgets.QPushButton(AboutMSUIDialog)
        self.btOK.setObjectName("btOK")
        self.horizontalLayout.addWidget(self.btOK)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.verticalLayout.addLayout(self.verticalLayout_2)

        self.retranslateUi(AboutMSUIDialog)
        self.btOK.clicked.connect(AboutMSUIDialog.accept)
        QtCore.QMetaObject.connectSlotsByName(AboutMSUIDialog)

    def retranslateUi(self, AboutMSUIDialog):
        _translate = QtCore.QCoreApplication.translate
        AboutMSUIDialog.setWindowTitle(_translate("AboutMSUIDialog", "About MSUI"))
        self.lblName.setText(_translate("AboutMSUIDialog", "Mission Support System User Interface"))
        self.lblPython.setText(_translate("AboutMSUIDialog", "Python Powered"))
        self.textBrowser.setHtml(_translate("AboutMSUIDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'DejaVu Sans Mono\'; color:#000000; background-color:#ffffff;\">Please read the reference documentation:</span><span style=\" font-family:\'DejaVu Sans Mono\'; color:#000000;\"><br /><br />Rautenhaus, M., Bauer, G., and Doernbrack, A.: A web service based tool to plan<br />atmospheric research flights, Geosci. Model Dev., 5,55-71, doi:10.5194/gmd-5-55-2012, 2012.<br /><br />and the paper\'s Supplement (which includes a tutorial) before using the application. The documents are available at:<br /><br /> * </span><span style=\" font-family:\'DejaVu Sans Mono\'; color:#000000; background-color:#ffffff;\">http://www.geosci-model-dev.net/5/55/2012/gmd-5-55-2012.pdf</span><span style=\" font-family:\'DejaVu Sans Mono\'; color:#000000;\"><br /> * http://www.geosci-model-dev.net/5/55/2012/gmd-5-55-2012-supplement.pdf<br /><br />When using this software, please be so kind and acknowledge its use by citing the above mentioned reference documentation in publications, presentations, reports, etc. that you create. Thank you very much.<br /><br /></span></p></body></html>"))
        self.lblVersion.setText(_translate("AboutMSUIDialog", "Version: --VERSION--"))
        self.lblChanges.setText(_translate("AboutMSUIDialog", "Changes:  --CHANGES--"))
        self.lblLicense.setText(_translate("AboutMSUIDialog", "License: Apache License Version 2.0"))
        self.lblCopyright.setText(_translate("AboutMSUIDialog", "Copyright 2008-2014: Deutsches Zentrum fuer Luft- und Raumfahrt e.V.\n"
"Copyright 2011-2014: Marc Rautenhaus\n"
"Copyright 2016-2021: by the mss team, see AUTHORS"))
        self.lblLinks.setText(_translate("AboutMSUIDialog", "<html><head/><body><p>See <a href=\"http://mss.rtfd.io\"><span style=\" text-decoration: underline; color:#0000ff;\">http://mss.rtfd.io</span></a> for detailed information and documentation.<br>Report bugs or feature requests at <a href=\"https://github.com/Open-MSS/MSS\"><span style=\" text-decoration: underline; color:#0000ff;\">https://github.com/Open-MSS/MSS</span></a>.</p></body></html>"))
        self.btOK.setText(_translate("AboutMSUIDialog", "Ok"))
